@@ -1,27 +1,70 @@
+import random
+from time import sleep
 
-def get_redis_instance(connection_pool=None):
-    REDIS_HOST = 'localhost'
-    REDIS_PORT = 6379
-    redis_instance = redis.Redis(
-        decode_responses=True,
-        host=REDIS_HOST,
-        port=REDIS_PORT,
-        db=0,
-        connection_pool=connection_pool
-    )
-    return redis_instance
+from .core import *
 
 
-def clean_db_after_test(redis_instance, prefix):
+def generate_token(chars_count):
+    allowed_chars = 'QWERTYUIOPASDFGHJKLZXCVBNM1234567890'
+    token = f'{"".join([random.choice(allowed_chars) for i in range(chars_count)])}'
+    return token
+
+
+def generate_token_12_chars():
+    return generate_token(12)
+
+
+class BotSession(RedisModel):
+    session_token = RedisString(default=generate_token_12_chars)
+    created = RedisDateTime(default=datetime.datetime.now)
+
+
+class TaskChallenge(RedisModel):
+    bot_session = RedisForeignKey(model=BotSession)
+    task_id = RedisNumber(default=0, null=False)
+    status = RedisString(default='in_work', choices={
+        'in_work': 'В работе',
+        'completed': 'Завершён успешно',
+        'completed_frozen_points': 'Завершён успешно, получил поинты в холде',
+        'completed_points': 'Завершён успешно, получил поинты',
+        'completed_decommissioning': 'Завершён успешно, поинты списаны',
+        'failed_bot': 'Зафейлил бот',
+        'failed_task_creator': 'Зафейлил создатель задания',
+    }, null=False)
+    account_checks_count = RedisNumber(default=0)
+    created = RedisDateTime(default=datetime.datetime.now)
+
+
+class TtlCheckModel(RedisModel):
+    redis_number_with_ttl = RedisNumber(default=0, null=False, ttl=5)
+
+
+class MetaTtlCheckModel(RedisModel):
+    redis_number = RedisNumber(default=0, null=False)
+    
+    class Meta:
+        ttl = 5
+
+
+class DictCheckModel(RedisModel):
+    redis_dict = RedisDict()
+
+
+class ListCheckModel(RedisModel):
+    redis_list = RedisList()
+
+
+def clean_db_after_test(connection_pool, prefix):
+    redis_instance = redis.Redis(connection_pool=connection_pool)
     for key in redis_instance.scan_iter(f'{prefix}:*'):
         redis_instance.delete(key)
 
 
-def basic_test(redis_instance, prefix):
+def basic_test(connection_pool, prefix):
     try:
         redis_root = RedisRoot(
             prefix=prefix,
-            redis_instance=redis_instance,
+            connection_pool=connection_pool,
             ignore_deserialization_errors=True
         )
         redis_root.register_models([
@@ -47,14 +90,14 @@ def basic_test(redis_instance, prefix):
                     have_exception = True
     except BaseException as ex:
         have_exception = True
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def auto_reg_test(redis_instance, prefix):
+def auto_reg_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     task_challenge_1 = TaskChallenge(
@@ -66,7 +109,7 @@ def auto_reg_test(redis_instance, prefix):
         have_exception = False
     except BaseException as ex:
         have_exception = True
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
@@ -88,10 +131,10 @@ def no_redis_instance_test(*args, **kwargs):
     return have_exception
 
 
-def choices_test(redis_instance, prefix):
+def choices_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     task_challenge_1 = TaskChallenge(
@@ -104,14 +147,14 @@ def choices_test(redis_instance, prefix):
         have_exception = True
     except BaseException as ex:
         have_exception = False
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def order_test(redis_instance, prefix):
+def order_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     for i in range(3):
@@ -127,14 +170,14 @@ def order_test(redis_instance, prefix):
             have_exception = False
     except BaseException as ex:
         pass
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def filter_test(redis_instance, prefix):
+def filter_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -152,15 +195,15 @@ def filter_test(redis_instance, prefix):
             have_exception = False
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def update_test(redis_instance, prefix):
+def update_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -176,15 +219,15 @@ def update_test(redis_instance, prefix):
                     have_exception = False
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def functions_like_defaults_test(redis_instance, prefix):
+def functions_like_defaults_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = False
@@ -195,15 +238,15 @@ def functions_like_defaults_test(redis_instance, prefix):
             have_exception = True
     except BaseException as ex:
         pass
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def redis_foreign_key_test(redis_instance, prefix):
+def redis_foreign_key_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -224,38 +267,15 @@ def redis_foreign_key_test(redis_instance, prefix):
                 have_exception = False
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-# def django_foreign_key_test(redis_instance, prefix):
-#     redis_root = RedisRoot(
-#         prefix=prefix,
-#         redis_instance=redis_instance,
-#         ignore_deserialization_errors=True
-#     )
-#     have_exception = True
-#     try:
-#         proxy = Proxy.objects.all()[0]
-#         django_foreign_key_model = DjangoForeignKeyModel(
-#             redis_root=redis_root,
-#             foreign_key=proxy,
-#         ).save()
-#         django_foreign_key_models = redis_root.get(DjangoForeignKeyModel)
-#         django_foreign_key_model = redis_root.order(django_foreign_key_models, '-id')[0]
-#         if django_foreign_key_model['foreign_key'] == proxy:
-#             have_exception = False
-#     except BaseException as ex:
-#         print(ex)
-#     clean_db_after_test(redis_instance, prefix)
-#     return have_exception
-
-
-def delete_test(redis_instance, prefix):
+def delete_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -275,43 +295,15 @@ def delete_test(redis_instance, prefix):
             have_exception = False
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def ttl_test(redis_instance, prefix):
+def save_consistency_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
-        ignore_deserialization_errors=True
-    )
-    have_exception = True
-    try:
-        ttl_check_model_1 = TtlCheckModel(
-            redis_root=redis_root,
-        ).save()
-        ttl_check_models = redis_root.get(TtlCheckModel)
-        if len(ttl_check_models):
-            ttl_check_model = ttl_check_models[0]
-            if 'redis_number_with_ttl' in ttl_check_model.keys():
-                sleep(6)
-                ttl_check_models = redis_root.get(TtlCheckModel)
-                if len(ttl_check_models):
-                    ttl_check_model = ttl_check_models[0]
-                    if 'redis_number_with_ttl' not in ttl_check_model.keys():
-                        have_exception = False
-    except BaseException as ex:
-        print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
-    return have_exception
-
-
-def save_consistency_test(redis_instance, prefix):
-    redis_root = RedisRoot(
-        prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True,
         save_consistency=True,
     )
@@ -332,15 +324,15 @@ def save_consistency_test(redis_instance, prefix):
                         have_exception = False
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def meta_ttl_test(redis_instance, prefix):
+def meta_ttl_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True,
     )
     have_exception = True
@@ -358,18 +350,18 @@ def meta_ttl_test(redis_instance, prefix):
                     have_exception = False
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def economy_test(redis_instance, prefix):
+def economy_test(connection_pool, prefix):
     have_exception = True
     try:
-
+        
         redis_root = RedisRoot(
             prefix=prefix,
-            redis_instance=redis_instance,
+            connection_pool=connection_pool,
             ignore_deserialization_errors=True,
             economy=True
         )
@@ -382,11 +374,11 @@ def economy_test(redis_instance, prefix):
             redis_root.update(TaskChallenge, task_challenge_1, account_checks_count=1)
         ended_in_economy = datetime.datetime.now()
         economy_time = (ended_in_economy - started_in_economy).total_seconds()
-        clean_db_after_test(redis_instance, prefix)
-
+        clean_db_after_test(connection_pool, prefix)
+        
         redis_root = RedisRoot(
             prefix=prefix,
-            redis_instance=redis_instance,
+            connection_pool=connection_pool,
             ignore_deserialization_errors=True,
             economy=False
         )
@@ -399,7 +391,7 @@ def economy_test(redis_instance, prefix):
             redis_root.update(TaskChallenge, task_challenge_1, account_checks_count=1)
         ended_in_no_economy = datetime.datetime.now()
         no_economy_time = (ended_in_no_economy - started_in_no_economy).total_seconds()
-        clean_db_after_test(redis_instance, prefix)
+        clean_db_after_test(connection_pool, prefix)
         economy_percent = round((no_economy_time / economy_time - 1) * 100, 2)
         economy_symbol = ('+' if economy_percent > 0 else '')
         print(f'Economy gives {economy_symbol}{economy_percent}% efficiency')
@@ -407,15 +399,15 @@ def economy_test(redis_instance, prefix):
             have_exception = False
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def dict_test(redis_instance, prefix):
+def dict_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -434,15 +426,15 @@ def dict_test(redis_instance, prefix):
                 have_exception = False
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def list_test(redis_instance, prefix):
+def list_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -458,27 +450,26 @@ def list_test(redis_instance, prefix):
                 have_exception = False
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def async_test(redis_instance, prefix):
+def async_test(connection_pool, prefix):
     have_exception = True
     try:
-
+        
         async def async_task(data_count, use_async=True):
             redis_roots = []
             connection_pool = redis.ConnectionPool(host='localhost', port=6379, db=0, decode_responses=True)
             for i in range(data_count):
-                redis_instance = get_redis_instance(connection_pool)
                 redis_root = RedisRoot(
                     prefix=prefix,
-                    redis_instance=redis_instance,
+                    connection_pool=connection_pool,
                     ignore_deserialization_errors=True
                 )
                 redis_roots.append(redis_root)
-
+            
             async def create_list(redis_roots, redis_root_index, list_length):
                 some_list = [random.choice('ABCDEF') for i in range(list_length)]
                 redis_root = redis_roots[redis_root_index]
@@ -490,7 +481,7 @@ def async_test(redis_instance, prefix):
                     'list': some_list,
                     'id': list_check_model_instance['id']
                 }
-
+            
             async def get_object_existence(redis_roots, redis_root_index, lists_with_ids):
                 exists = False
                 redis_root = redis_roots[redis_root_index]
@@ -499,7 +490,7 @@ def async_test(redis_instance, prefix):
                     if len(objects) == 1:
                         exists = objects[0]['id'] == lists_with_ids['id']
                 return exists
-
+            
             async def update_list(redis_roots, redis_root_index, lists_with_ids):
                 result = False
                 redis_root = redis_roots[redis_root_index]
@@ -515,7 +506,7 @@ def async_test(redis_instance, prefix):
                             if len(updated_objects) == 1:
                                 result = updated_objects[0]['id'] == lists_with_ids['id']
                 return result
-
+            
             async def delete_list(redis_roots, redis_root_index, id):
                 result = False
                 redis_root = redis_roots[redis_root_index]
@@ -528,7 +519,7 @@ def async_test(redis_instance, prefix):
                         if not objects:
                             result = True
                 return result
-
+            
             async def create_lists(lists_count, use_async=True):
                 if use_async:
                     async_create_lists_tasks = [
@@ -542,7 +533,7 @@ def async_test(redis_instance, prefix):
                         for i in range(lists_count)
                     ]
                 return list_of_lists_with_ids
-
+            
             async def get_objects_existence(list_of_lists_with_ids, use_async=True):
                 if use_async:
                     async_get_object_by_list_tasks = [
@@ -556,7 +547,7 @@ def async_test(redis_instance, prefix):
                         for i, lists_with_ids in enumerate(list_of_lists_with_ids)
                     ]
                 return list_of_results
-
+            
             async def update_lists(list_of_lists_with_ids, use_async=True):
                 if use_async:
                     async_update_list_tasks = [
@@ -570,7 +561,7 @@ def async_test(redis_instance, prefix):
                         for i, lists_with_ids in enumerate(list_of_lists_with_ids)
                     ]
                 return list_of_results
-
+            
             async def delete_lists(list_of_lists_with_ids, use_async=True):
                 if use_async:
                     async_delete_list_tasks = [
@@ -584,9 +575,7 @@ def async_test(redis_instance, prefix):
                         for i, lists_with_ids in enumerate(list_of_lists_with_ids)
                     ]
                 return list_of_results
-
-
-
+            
             list_of_lists_with_ids = await create_lists(data_count, use_async)
             list_of_results = await get_objects_existence(list_of_lists_with_ids, use_async)
             if all(list_of_results):
@@ -594,7 +583,7 @@ def async_test(redis_instance, prefix):
                 if all(list_of_results):
                     list_of_results = await delete_lists(list_of_lists_with_ids, use_async)
             return all(list_of_results)
-
+        
         data_count = 10
         async_started_in = datetime.datetime.now()
         async_result = asyncio.run(async_task(data_count))
@@ -605,20 +594,25 @@ def async_test(redis_instance, prefix):
         sync_result = asyncio.run(async_task(data_count, False))
         sync_ended_in = datetime.datetime.now()
         sync_time = (sync_ended_in - sync_started_in).total_seconds()
-
+        
         async_percent = round((async_time / sync_time - 1) * 100, 2)
         async_symbol = ('+' if async_percent > 0 else '')
         print(f'Async gives {async_symbol}{async_percent}% efficiency')
-
+    
     except BaseException as ex:
         print(ex)
-
-    clean_db_after_test(redis_instance, prefix)
+    
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
 def run_tests():
-    redis_instance = get_redis_instance()
+    connection_pool = redis.ConnectionPool(
+        host='localhost',
+        port=6379,
+        db=0,
+        decode_reponses=True
+    )
     tests = [
         basic_test,
         auto_reg_test,
@@ -628,10 +622,8 @@ def run_tests():
         filter_test,
         functions_like_defaults_test,
         redis_foreign_key_test,
-        # django_foreign_key_test,
         update_test,
         delete_test,
-        ttl_test,
         save_consistency_test,
         meta_ttl_test,
         economy_test,
@@ -645,7 +637,7 @@ def run_tests():
     for i, test in enumerate(tests):
         print(f'Starting {int(i + 1)} test: {test.__name__.replace("_", " ")}')
         test_started_in = datetime.datetime.now()
-        result = not test(redis_instance, test.__name__)
+        result = not test(connection_pool, test.__name__)
         test_ended_in = datetime.datetime.now()
         test_time = (test_ended_in - test_started_in).total_seconds()
         print(f'{result = } / {test_time}s\n')
